/**
 * Class JGraphEditor. 
 * 
 * @version 20021102
 *
 * @author Andrea Valente
 * Member of <br><b>T</b>he <b>A</b>vatars <b>G</b>roup
 * <br>sf.net/projects/avatarsdev
 */

package sge;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

public class JGraphEditor extends JPanel {
Vector nodes;
Vector arcs;
int posX,posY;
Item selected=null;
boolean selectedIsNode=false;

boolean dragDrop=false;
int dDx,dDy;
int nam=0; // new arc mode. 
					// 0=not active,1=selecting fromNode, 2=selecting toNode.
int namFromId=0;

static BasicStroke dash = new BasicStroke(1, BasicStroke.CAP_ROUND, 
						BasicStroke.JOIN_ROUND, 0, new float[]{6,6}, 0);
						
	JGraphEditor(){
		//JLabel l= new JLabel("JGraphEditor");
		//add(l);
		setBackground( Color.white );
		reshape(0,0,1000,1000);
		setPreferredSize(new Dimension(1000,1000) );

		nodes=new Vector();
		arcs=new Vector();
		posX=10;
		posY=10;

		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent me) {
				//System.out.println("DOWN at : "+me.getX() + ", " + me.getY());
				dragDrop=false;
				if (selected!=null) {
					if (selectedIsNode){
						Node sn=(Node)selected;
						if ( sn.isInside(me.getX(),me.getY()) ){
							dragDrop=true;
							dDx=me.getX()-sn.getX();
							dDy=me.getY()-sn.getY();
						}
					}
				}
			}
			
			public void mouseReleased(MouseEvent me) {
				//System.out.println("UP at : "+me.getX() + ", " + me.getY());
				if (dragDrop){
					if (selectedIsNode){
						Node sn=(Node)selected;
						sn.setPosition(me.getX()-dDx,me.getY()-dDy);
						repaint();
					}
				}
				dragDrop=false;
			}

			public void mouseClicked(MouseEvent me) {
				//System.out.println("CLICKED at : "+me.getX() + ", " + me.getY());

				// Check for selection of a node
				Node n=null;
				deSelect();
				for (int i=0;((i<nodes.size()) & (selected==null));i++){
					n=(Node) nodes.elementAt(i);
					if ( n.isInside(me.getX(),me.getY()) ){
						select(n);
					}
				}

				if (selected!=null){
					if (selectedIsNode){
						Node sn=(Node)selected;
						if (nam==1){
							namFromId=sn.getId();
							nam=2;
							//deSelect();
						} else if (nam==2){
							addArc(namFromId,sn.getId());
							nam=0;
							//deSelect();
						}
					}
				} else {
					// Check for selection of a arcs
					Arc a=null;
					for (int i=0;((i<arcs.size()) & (selected==null));i++){
						a=(Arc) arcs.elementAt(i);
						if ( a.isInside(  me.getX(),me.getY(),
									getNode( a.getFromId() ), 
									getNode( a.getToId()   ) ) 
							){
							select(a);
						}
					}
				} // end Selection arcs
				
				((JComponent) me.getSource() ).repaint();
			}
		});

	}

// --------------- Actions ----------------------------- begin

	public void
	newNode(){
		nodes.add( new Node(posX,posY) );
		posX+=10;
		posY+=15;
		repaint();
	}

	// Delete currently selected node
	public void
	delNode(){
		if (selected!=null){
			if (selectedIsNode){
				Node sn=(Node)selected;
				int id=sn.getId();
				nodes.remove(sn);
				deSelect();
				// remove dangling arcs
				for (int i=0;i<arcs.size();i++){
					Arc a=(Arc)arcs.elementAt(i);
					if ( (a.getFromId()==id) | (a.getToId()==id) ){
						arcs.remove(a);
						i--;
					}
				}
				repaint();
			}
		}
	}

	// Delete currently selected arc
	public void
	delArc(){
		if (selected!=null){
			if (!selectedIsNode){
				Arc a=(Arc)selected;
				arcs.remove(a);
				deSelect();
				repaint();
			}
		}
	}


	public void
	newArcMode(){
		nam=1;
		deSelect();
	}

// --------------- Actions ----------------------------- end

	private void
	addArc(int fromId,int toId){
		arcs.add( new Arc(fromId,toId) );
		repaint();
	
	}
	 
	private void
	select(Item it){
		selected=it;
		selectedIsNode=false;
		if (it instanceof Node)
			selectedIsNode=true;
	}
	 
	private void
	select(Node n){
		selected=n;
		selectedIsNode=true;
	}

	private void
	select(Arc a){
		selected=a;
		selectedIsNode=false;
	}

	private void
	deSelect(){
		selected=null;
		selectedIsNode=false;
	}
	
	private Node
	getNode(int id){
		Node.DUMMY.setId(id);
		int index=nodes.indexOf(Node.DUMMY);
		Node node=(Node) nodes.elementAt(index);
		return node;	
	}

	public void 
	paintComponent(Graphics gr) {
		super.paintComponent(gr);      //clears the background
		Graphics2D g=(Graphics2D)gr;

		Stroke str=g.getStroke();

		// Draw grid
		Insets insets = getInsets();
		int currentWidth = getWidth() - insets.left - insets.right;
		int currentHeight = getHeight() - insets.top - insets.bottom;
		int cw=currentWidth-1;
		int ch=currentHeight-1;

		int stepx=15,stepy=15;
		double kx=currentWidth/(0.0+stepx),ky=currentHeight/(0.0+stepy);
		g.setColor(Color.cyan);
		g.setStroke(dash);		
		for (int i=0;i<stepx;i++){
			g.drawLine((int)(i*kx),0,(int)(i*kx),ch);
			g.drawLine(0,(int)(i*ky),cw,(int)(i*ky));
		}
		g.setStroke(str);

		// arcs and nodes 
		g.setColor(Color.black);
		Arc  a=null;
		for (int i=0;i<arcs.size();i++){
			a=(Arc) arcs.elementAt(i);

			Node n1=getNode(a.getFromId()),
				n2=getNode(a.getToId());
			int r1=n1.getRadius();
			int r2=n2.getRadius();
			int x1=n1.getX()+n1.getW()/2,
				y1=n1.getY()+n1.getH()/2,
				x2=n2.getX()+n2.getW()/2,
				y2=n2.getY()+n2.getH()/2;
			
			// draw arc
			g.setColor(Color.black);
			g.drawLine(x1,y1, x2,y2);

			// P(t) = (1-t) * A + t * B  ,  t in [0,1]
			double t=3.0/4.0;
			double xd1=x1,
					yd1=y1,
					xd2=x2,
					yd2=y2;
			g.fillOval( (int) ( (1-t)*xd1 + t*xd2 ) -3,
							(int) ( (1-t)*yd1 + t*yd2 ) -3, 
							5,5);

			if (a==selected){ // draw bounding box
				g.setColor(Color.gray);
				if (x1>x2){
					int tmp=x1;
					x1=x2;
					x2=tmp;
				}
				if (y1>y2){
					int tmp=y1;
					y1=y2;
					y2=tmp;
				}
				g.drawRect(x1,y1, x2-x1,y2-y1);
				g.setColor(Color.black);
			}

			g.setStroke(str);
		}

		Node n=null;
		for (int i=0;i<nodes.size();i++){
			n=(Node) nodes.elementAt(i);
			if (n==selected){ // draw bounding box
				g.setColor(Color.gray);
				g.drawRect(n.getX()-3,n.getY()-3,n.getW()+3,n.getH()+3);
				g.setColor(Color.black);
			}
			// clear background
			g.setColor(Color.white);
			g.fillOval(n.getX()-1,n.getY()-1,n.getW(),n.getH());
			// draw thick node
			g.setColor(Color.black);
			g.setStroke(new BasicStroke(2));
			g.drawOval(n.getX()-1,n.getY()-1,n.getW(),n.getH());
			g.setStroke(str);
		}

	}

	public String
	toString(){
      return "aJGraphEditor";
	}

	public void
	saveGraph(){
		JFrame f=new JFrame();
		FileDialog fd = new FileDialog(f,"Saving graph", FileDialog.SAVE);
		fd.show();

		String fName=fd.getFile();
		if (fName!=null){

			try{
				FileOutputStream fout;
				fout=new FileOutputStream( fName );
				ObjectOutputStream oos = new ObjectOutputStream( fout );
				oos.writeObject( new Integer(Node.NODEID) );
				oos.writeObject( nodes );
				oos.writeObject( arcs );
			} catch(Exception e){
				System.out.println("Errore durante salvataggio!");
			}
		} // end IF
	}

	public void
	loadGraph(){
		JFrame f=new JFrame();
		FileDialog fd = new FileDialog(f,"Loading graph", FileDialog.LOAD);
		fd.show();
		String fName=fd.getFile();
		if (fName!=null){
			Vector v=null,v1=null;
			Integer num=null;
			try{
				FileInputStream fin;
				fin=new FileInputStream( fName ); 
				ObjectInputStream ois = new ObjectInputStream(fin);
				num=(Integer)ois.readObject();
				v=(Vector)ois.readObject();
				v1=(Vector)ois.readObject();
				//System.out.println( v );
			} catch(Exception e){
				System.out.println("Errore durante caricamento!");
			}
			if (v!=null){
				dragDrop=false;
				deSelect();
				Node.NODEID=num.intValue();
				nodes=v;
				arcs=v1;
				repaint();
			}
		} // end IF
	}

}
